package cn.iocoder.yudao.module.basic.service.businessinfo;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.module.basic.controller.admin.businessinfo.vo.*;
import cn.iocoder.yudao.module.basic.dal.dataobject.businessinfo.BusinessInfoDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 商户 Service 接口
 *
 * @author 芋道源码
 */
public interface BusinessInfoService {

    /**
     * 创建商户
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createBusinessInfo(@Valid BusinessInfoCreateReqVO createReqVO);

    /**
     * 更新商户
     *
     * @param updateReqVO 更新信息
     */
    void updateBusinessInfo(@Valid BusinessInfoUpdateReqVO updateReqVO);

    /**
     * 删除商户
     *
     * @param id 编号
     */
    CommonResult<Boolean> deleteBusinessInfo(Long id);

    /**
     * 获得商户
     *
     * @param id 编号
     * @return 商户
     */
    BusinessInfoDO getBusinessInfo(Long id);

    /**
     * 获得商户名称列表
     *
     * @return 商户名称列表
     */
    List<BusinessInfoDO> getBusinessNameList();

    /**
     * 获得商户分页
     *
     * @param pageReqVO 分页查询
     * @return 商户分页
     */
    PageResult<BusinessInfoDO> getBusinessInfoPage(BusinessInfoPageReqVO pageReqVO);

    /**
     * 根据id,校验商户是否存在
     *
     * @param id 查询条件
     */
    void validateBusinessInfoExists(Long id);

    /**
     * 根据商户编码获取商户信息
     * @param businessId
     * @return
     */
    BusinessInfoDO getBusinessInfoByBusinessId(String businessId);

}
