package cn.iocoder.yudao.module.basic.service.dealergoods;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.basic.controller.admin.dealergoods.vo.*;
import cn.iocoder.yudao.module.basic.dal.dataobject.dealergoods.DealerGoodsDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 经销商经销产品 Service 接口
 *
 * @author 芋道源码
 */
public interface DealerGoodsService {

    /**
     * 创建经销商经销产品
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createDealerGoods(@Valid DealerGoodsCreateReqVO createReqVO);

    /**
     * 更新经销商经销产品
     *
     * @param updateReqVO 更新信息
     */
    void updateDealerGoods(@Valid DealerGoodsUpdateReqVO updateReqVO);

    /**
     * 删除经销商经销产品
     *
     * @param id 编号
     */
    void deleteDealerGoods(Long id);

    /**
     * 获得经销商经销产品
     *
     * @param id 编号
     * @return 经销商经销产品
     */
    DealerGoodsDO getDealerGoods(Long id);

    /**
     * 获得经销商经销产品列表
     *
     * @param dealerGoodsListReqVO 根据信息内容动态查询
     * @return 经销商经销产品列表
     */
    List<DealerGoodsDO> getDealerGoodsListQuery(DealerGoodsListReqVO dealerGoodsListReqVO);
    /**
     * 获得经销商经销产品分页
     *
     * @param pageReqVO 分页查询
     * @return 经销商经销产品分页
     */
    PageResult<DealerGoodsDO> getDealerGoodsPage(DealerGoodsPageReqVO pageReqVO);

    /**
     * 根据商品ID和经销商ID获取所有对应的销售区域
     * @param dealerId
     * @return
     */
    List<DealerGoodsDO> getSaleAreasByGoodsAndDealer(String goodsId, String dealerId) ;
}
