package cn.iocoder.yudao.module.basic.service.unit;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import cn.iocoder.yudao.module.basic.controller.admin.unit.vo.*;
import cn.iocoder.yudao.module.basic.dal.dataobject.unit.UnitDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import cn.iocoder.yudao.module.basic.convert.unit.UnitConvert;
import cn.iocoder.yudao.module.basic.dal.mysql.unit.UnitMapper;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.basic.enums.ErrorCodeConstants.*;

/**
 * 单位 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class UnitServiceImpl implements UnitService {

    @Resource
    private UnitMapper unitMapper;

    @Override
    public Long createUnit(UnitCreateReqVO createReqVO) {
        //校验单位昵称是否重复
        validateUnitNameExists(createReqVO.getUnitName());
        // 插入
        UnitDO unit = UnitConvert.INSTANCE.convert(createReqVO);
        unitMapper.insert(unit);
        // 返回
        return unit.getId();
    }

    private void validateUnitNameExists(String unitName) {
        QueryWrapper<UnitDO> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("unit_name",unitName);
        if (unitMapper.selectOne(queryWrapper) != null){
            throw exception(UNIT_NAME_ALREADY_EXISTS);
        }
    }

    @Override
    public void updateUnit(UnitUpdateReqVO updateReqVO) {
        // 校验存在
        UnitDO unitInfo = unitMapper.selectById(updateReqVO.getId());
        if (unitInfo == null){
            throw exception(UNIT_NOT_EXISTS);
        }
        // 检查单位名称是否发生变化
        if (!Objects.equals(unitInfo.getUnitName(),updateReqVO.getUnitName())) {
            //校验商户昵称的地块类型名称是否重复
            validateUnitNameExists(updateReqVO.getUnitName());
        }
        // 更新
        UnitDO updateObj = UnitConvert.INSTANCE.convert(updateReqVO);
        unitMapper.updateById(updateObj);
    }

    @Override
    public void deleteUnit(Long id) {
        // 校验存在
        validateUnitExists(id);
        // 删除
        unitMapper.deleteById(id);
    }

    private void validateUnitExists(Long id) {
        if (unitMapper.selectById(id) == null) {
            throw exception(UNIT_NOT_EXISTS);
        }
    }

    @Override
    public UnitDO getUnit(Long id) {
        return unitMapper.selectById(id);
    }

    @Override
    public List<UnitDO> getUnitList(Collection<Long> ids) {
        return unitMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<UnitDO> getUnitPage(UnitPageReqVO pageReqVO) {
        return unitMapper.selectPage(pageReqVO);
    }

}
