package cn.iocoder.yudao.module.basic.service.workersinfo;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.basic.controller.admin.workersinfo.vo.*;
import cn.iocoder.yudao.module.basic.controller.app.workersinfo.vo.Code2SessionReqVO;
import cn.iocoder.yudao.module.basic.controller.app.workersinfo.vo.VerifyReqVO;
import cn.iocoder.yudao.module.basic.dal.dataobject.workersinfo.WorkersInfoDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 员工 Service 接口
 *
 * @author 芋道源码
 */
public interface WorkersInfoService {

    /**
     * 创建员工
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createWorkersInfo(@Valid WorkersInfoCreateReqVO createReqVO);

    /**
     * 更新员工
     *
     * @param updateReqVO 更新信息
     */
    void updateWorkersInfo(@Valid WorkersInfoUpdateReqVO updateReqVO);

    /**
     * 删除员工
     *
     * @param id 编号
     */
    void deleteWorkersInfo(Long id);

    /**
     * 获得员工
     *
     * @param id 编号
     * @return 员工
     */
    WorkersInfoDO getWorkersInfo(Long id);

    /**
     * 获得员工分页
     *
     * @param pageReqVO 分页查询
     * @return 员工分页
     */
    PageResult<WorkersInfoDO> getWorkersInfoPage(WorkersInfoPageReqVO pageReqVO);

    /**
     * 条件查询 根据商户id 查询所有底下的员工昵称
     * @param belongBusinessId
     * @return
     */
    List<WorkersInfoDO> findByBelongBusinessId(Long belongBusinessId);

    /**
     * 根据商户id 手机号获取员工信息
     * @param businessInfoId 商户ID
     * @param openid openid
     * @return 员工信息
     */
    WorkersInfoDO getWorkersInfo(Long businessInfoId, String openid);

    /**
     * 绑定微信
     * @param reqVO 入参
     * @return openid
     */
    String bindWXToWorkers(Code2SessionReqVO reqVO);

    /**
     * 解绑微信
     * @param id id
     */
    void unbindingWX(Long id);

    /**
     * 根据openid获得员工， 如果已绑定openid，返回员工信息，否则返回null
     * @param reqVO 入参
     * @return 员工信息
     */
    WorkersInfoDO getWorkersInfoByOpenId(VerifyReqVO reqVO);
}
