package cn.iocoder.yudao.module.configure.service.inspectiontemplate;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.configure.controller.admin.inspectiontemplate.vo.InspectionTemplateCreateReqVO;
import cn.iocoder.yudao.module.configure.controller.admin.inspectiontemplate.vo.InspectionTemplatePageReqVO;
import cn.iocoder.yudao.module.configure.controller.admin.inspectiontemplate.vo.InspectionTemplateRespVO;
import cn.iocoder.yudao.module.configure.controller.admin.inspectiontemplate.vo.InspectionTemplateUpdateReqVO;
import cn.iocoder.yudao.module.configure.dal.dataobject.inspectiontemplate.InspectionTemplateDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 检测报告模板 Service 接口
 *
 * @author 芋道源码
 */
public interface InspectionTemplateService {

    /**
     * 创建检测报告模板
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createInspectionTemplate(@Valid InspectionTemplateCreateReqVO createReqVO);

    /**
     * 更新检测报告模板
     *
     * @param updateReqVO 更新信息
     */
    void updateInspectionTemplate(@Valid InspectionTemplateUpdateReqVO updateReqVO);

    /**
     * 删除检测报告模板
     *
     * @param id 编号
     */
    void deleteInspectionTemplate(Long id);

    /**
     * 获得检测报告模板
     *
     * @param id 编号
     * @return 检测报告模板
     */
    InspectionTemplateRespVO getInspectionTemplate(Long id);

    /**
     * 获得检测报告模板分页
     *
     * @param pageReqVO 分页查询
     * @return 检测报告模板分页
     */
    PageResult<InspectionTemplateDO> getInspectionTemplatePage(InspectionTemplatePageReqVO pageReqVO);

    /**
     * 根据商户ID获取检测报告模板列表
     * @param businessId 商户ID
     * @return 检测报告模板
     */
    List<InspectionTemplateRespVO> getInspectionTemplateByBusinessId(Long businessId);

}
