package cn.iocoder.yudao.module.configure.service.materialmanagement;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.configure.controller.admin.materialmanagement.vo.MaterialManagementCreateReqVO;
import cn.iocoder.yudao.module.configure.controller.admin.materialmanagement.vo.MaterialManagementGroupUpdate;
import cn.iocoder.yudao.module.configure.controller.admin.materialmanagement.vo.MaterialManagementPageReqVO;
import cn.iocoder.yudao.module.configure.controller.admin.materialmanagement.vo.MaterialManagementUpdateReqVO;
import cn.iocoder.yudao.module.configure.dal.dataobject.materialmanagement.MaterialManagementDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 素材管理 Service 接口
 *
 * @author 芋道源码
 */
public interface MaterialManagementService {

    /**
     * 创建素材管理
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Boolean createMaterialManagement(@Valid List<MaterialManagementCreateReqVO> createReqVO);

    /**
     * 更新素材管理
     *
     * @param updateReqVO 更新信息
     */
    void updateMaterialManagement(@Valid MaterialManagementUpdateReqVO updateReqVO);

    /**
     * 批量移动素材管理分组
     *
     */
    void updateMaterialManagementGroupBatch(MaterialManagementGroupUpdate materialManagementGroupUpdate);

    /**
     * 删除素材管理
     *
     * @param id 编号
     */
    void deleteMaterialManagement(Long id);

    /**
     * 批量删除素材管理
     *
     * @param ids 编号
     */
    void deleteMaterialManagementBatch(List<Long> ids);

    /**
     * 获得素材管理
     *
     * @param id 编号
     * @return 素材管理
     */
    MaterialManagementDO getMaterialManagement(Long id);

    /**
     * 获得素材管理列表
     *
     * @param ids 编号
     * @return 素材管理列表
     */
    List<MaterialManagementDO> getMaterialManagementList(Collection<Long> ids);

    /**
     * 获得素材管理分页
     *
     * @param pageReqVO 分页查询
     * @return 素材管理分页
     */
    PageResult<MaterialManagementDO> getMaterialManagementPage(MaterialManagementPageReqVO pageReqVO);

}
