package cn.iocoder.yudao.module.configure.service.unitemplate;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;
import cn.iocoder.yudao.module.configure.controller.admin.unitemplate.vo.UniTemplateCreateReqVO;
import cn.iocoder.yudao.module.configure.controller.admin.unitemplate.vo.UniTemplateUpdateReqVO;
import cn.iocoder.yudao.module.configure.dal.dataobject.unitemplate.UniTemplateDO;
import cn.iocoder.yudao.module.configure.dal.mysql.unitemplate.UniTemplateMapper;
import org.junit.jupiter.api.Test;
import org.springframework.context.annotation.Import;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertPojoEquals;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertServiceException;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomLongId;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomPojo;
import static cn.iocoder.yudao.module.configure.enums.ErrorCodeConstants.UNI_TEMPLATE_NOT_EXISTS;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;

/**
* {@link UniTemplateServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(UniTemplateServiceImpl.class)
public class UniTemplateServiceImplTest extends BaseDbUnitTest {

    @Resource
    private UniTemplateServiceImpl uniTemplateService;

    @Resource
    private UniTemplateMapper uniTemplateMapper;

    @Test
    public void testCreateUniTemplate_success() {
        // 准备参数
        UniTemplateCreateReqVO reqVO = randomPojo(UniTemplateCreateReqVO.class);

        // 调用
        Long uniTemplateId = uniTemplateService.createUniTemplate(reqVO);
        // 断言
        assertNotNull(uniTemplateId);
        // 校验记录的属性是否正确
        UniTemplateDO uniTemplate = uniTemplateMapper.selectById(uniTemplateId);
        assertPojoEquals(reqVO, uniTemplate);
    }

    @Test
    public void testUpdateUniTemplate_success() {
        // mock 数据
        UniTemplateDO dbUniTemplate = randomPojo(UniTemplateDO.class);
        uniTemplateMapper.insert(dbUniTemplate);// @Sql: 先插入出一条存在的数据
        // 准备参数
        UniTemplateUpdateReqVO reqVO = randomPojo(UniTemplateUpdateReqVO.class, o -> {
            o.setId(dbUniTemplate.getId()); // 设置更新的 ID
        });

        // 调用
        uniTemplateService.updateUniTemplate(reqVO);
        // 校验是否更新正确
        UniTemplateDO uniTemplate = uniTemplateMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, uniTemplate);
    }

    @Test
    public void testUpdateUniTemplate_notExists() {
        // 准备参数
        UniTemplateUpdateReqVO reqVO = randomPojo(UniTemplateUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> uniTemplateService.updateUniTemplate(reqVO), UNI_TEMPLATE_NOT_EXISTS);
    }

    @Test
    public void testDeleteUniTemplate_success() {
        // mock 数据
        UniTemplateDO dbUniTemplate = randomPojo(UniTemplateDO.class);
        uniTemplateMapper.insert(dbUniTemplate);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbUniTemplate.getId();

        // 调用
        uniTemplateService.deleteUniTemplate(id);
       // 校验数据不存在了
       assertNull(uniTemplateMapper.selectById(id));
    }

    @Test
    public void testDeleteUniTemplate_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> uniTemplateService.deleteUniTemplate(id), UNI_TEMPLATE_NOT_EXISTS);
    }

}
