package cn.iocoder.yudao.module.identity.service.diversionrecord;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.identity.controller.admin.diversionrecord.vo.DiversionRecordExportReqVO;
import cn.iocoder.yudao.module.identity.controller.admin.diversionrecord.vo.DiversionRecordPageReqVO;
import cn.iocoder.yudao.module.identity.controller.admin.diversionrecord.vo.DiversionRecordUpdateReqVO;
import cn.iocoder.yudao.module.identity.dal.dataobject.code.CodeDO;
import cn.iocoder.yudao.module.identity.dal.dataobject.diversionrecord.DiversionRecordDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 疑似窜货记录 Service 接口
 *
 * @author 芋道源码
 */
public interface DiversionRecordService {

    /**
     * 更新疑似窜货记录如果是窜货那就新增窜货数据
     *
     * @param updateReqVO 更新信息
     */
    void updateDiversionRecord(@Valid DiversionRecordUpdateReqVO updateReqVO);

    /**
     * 删除疑似窜货记录
     *
     * @param id 编号
     */
    void deleteDiversionRecord(Long id);

    /**
     * 获得疑似窜货记录
     *
     * @param id 编号
     * @return 疑似窜货记录
     */
    DiversionRecordDO getDiversionRecord(Long id);

    /**
     * 获得疑似窜货记录列表
     *
     * @param ids 编号
     * @return 疑似窜货记录列表
     */
    List<DiversionRecordDO> getDiversionRecordList(Collection<Long> ids);

    /**
     * 获得疑似窜货记录分页
     *
     * @param pageReqVO 分页查询
     * @return 疑似窜货记录分页
     */
    PageResult<DiversionRecordDO> getDiversionRecordPage(DiversionRecordPageReqVO pageReqVO);

    /**
     * 获得疑似窜货记录列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 疑似窜货记录列表
     */
    List<DiversionRecordDO> getDiversionRecordList(DiversionRecordExportReqVO exportReqVO);

    /**
     * 根据消费者扫码地区和经销商销售区域的对比结果自动生成
     *
     * @param codeDO  标识码对象
     * @return true表示是窜货，false表示不是窜货
     */
    void generateDiversionRecord(CodeDO codeDO, String scanLocation);

    /**
     * 根据商品编码查询所有数据 然后在查询出所有的数据的同时再过滤出所有相对应的扫码地区
     * @param goodsId
     * @return
     */
    List<DiversionRecordDO> getScanLocationsAndTimesByGoodsId(String goodsId, String batchId, String scanLocation);

    /**
     * 查询出所有一致的商品编码和扫码地区
     * @param records
     * @param batchId
     * @param scanLocation
     * @return
     */
    List<String> getConsistentGoodsIdAndScanLocations(List<DiversionRecordDO> records, String batchId, String scanLocation);
}
