package cn.iocoder.yudao.module.identity.service.diversionstats;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.identity.controller.admin.diversionstats.vo.*;
import cn.iocoder.yudao.module.identity.dal.dataobject.diversionstats.DiversionStatsDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 窜货统计 Service 接口
 *
 * @author 芋道源码
 */
public interface DiversionStatsService {

    /**
     * 创建窜货统计
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createDiversionStats(@Valid DiversionStatsCreateReqVO createReqVO);

    /**
     * 更新窜货统计
     *
     * @param updateReqVO 更新信息
     */
    void updateDiversionStats(@Valid DiversionStatsUpdateReqVO updateReqVO);

    /**
     * 删除窜货统计
     *
     * @param id 编号
     */
    void deleteDiversionStats(Long id);

    /**
     * 获得窜货统计
     *
     * @param id 编号
     * @return 窜货统计
     */
    DiversionStatsDO getDiversionStats(Long id);

    /**
     * 获得窜货统计列表
     *
     * @param ids 编号
     * @return 窜货统计列表
     */
    List<DiversionStatsDO> getDiversionStatsList(Collection<Long> ids);

    /**
     * 获得窜货统计分页
     *
     * @param pageReqVO 分页查询
     * @return 窜货统计分页
     */
    PageResult<DiversionStatsDO> getDiversionStatsPage(DiversionStatsPageReqVO pageReqVO);

    /**
     * 获得窜货统计列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 窜货统计列表
     */
    List<DiversionStatsDO> getDiversionStatsList(DiversionStatsExportReqVO exportReqVO);

}
