package cn.iocoder.yudao.module.identity.service.generatebatch;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.identity.controller.admin.generatebatch.vo.*;
import cn.iocoder.yudao.module.identity.dal.dataobject.generatebatch.GenerateBatchDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 码生成记录与批次关联记录 Service 接口
 *
 * @author 芋道源码
 */
public interface GenerateBatchService {

    /**
     * 创建码生成记录与批次关联记录
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createGenerateBatch(@Valid GenerateBatchCreateReqVO createReqVO);

    /**
     * 更新码生成记录与批次关联记录
     *
     * @param updateReqVO 更新信息
     */
    void updateGenerateBatch(@Valid GenerateBatchUpdateReqVO updateReqVO);

    /**
     * 删除码生成记录与批次关联记录
     *
     * @param id 编号
     */
    void deleteGenerateBatch(Long id);

    /**
     * 获得码生成记录与批次关联记录
     *
     * @param id 编号
     * @return 码生成记录与批次关联记录
     */
    GenerateBatchDO getGenerateBatch(Long id);

    /**
     * 获得码生成记录与批次关联记录列表
     *
     * @param ids 编号
     * @return 码生成记录与批次关联记录列表
     */
    List<GenerateBatchDO> getGenerateBatchList(Collection<Long> ids);

    /**
     * 获得码生成记录与批次关联记录分页
     *
     * @param pageReqVO 分页查询
     * @return 码生成记录与批次关联记录分页
     */
    PageResult<GenerateBatchDO> getGenerateBatchPage(GenerateBatchPageReqVO pageReqVO);

    /**
     * 获得码生成记录与批次关联记录列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 码生成记录与批次关联记录列表
     */
    List<GenerateBatchDO> getGenerateBatchList(GenerateBatchExportReqVO exportReqVO);

    /**
     * 根据码生成记录ID获取码生成记录与批次关联记录数据信息
     * @param generateRecordId
     * @return
     */
    GenerateBatchDO getGenerateBatchByRecordId(String generateRecordId);

}
