package cn.iocoder.yudao.module.identity.service.generaterecord;

import java.io.IOException;
import java.util.*;
import javax.servlet.http.HttpServletResponse;
import javax.validation.*;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.module.identity.controller.admin.generaterecord.vo.*;
import cn.iocoder.yudao.module.identity.dal.dataobject.generaterecord.GenerateRecordDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import com.google.zxing.WriterException;

/**
 * 标识码生成记录 Service 接口
 *
 * @author 芋道源码
 */
public interface GenerateRecordService {

    /**
     * 创建标识码生成记录
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createGenerateRecord(@Valid GenerateRecordCreateReqVO createReqVO);
    /**
     * 更新标识码生成记录
     *
     * @param updateReqVO 更新信息
     */
    void updateGenerateRecord(@Valid GenerateRecordUpdateReqVO updateReqVO);

    /**
     * 删除标识码生成记录
     *
     * @param id 编号
     */
    void deleteGenerateRecord(Long id);

    /**
     * 获得标识码生成记录
     *
     * @param id 编号
     * @return 标识码生成记录
     */
    GenerateRecordDO getGenerateRecord(Long id);

    /**
     * 获得标识码生成记录列表
     *
     * @param ids 编号
     * @return 标识码生成记录列表
     */
    List<GenerateRecordDO> getGenerateRecordList(Collection<Long> ids);

    /**
     * 获得标识码生成记录分页
     *
     * @param pageReqVO 分页查询
     * @return 标识码生成记录分页
     */
    PageResult<GenerateRecordRespVO> getGenerateRecordPage(GenerateRecordPageReqVO pageReqVO);

    /**
     * 获得标识码生成记录列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 标识码生成记录列表
     */
    List<GenerateRecordDO> getGenerateRecordList(GenerateRecordExportReqVO exportReqVO);

    /**
     * 根据 generateRecordId记录编码查询出该条数据信息
     * @param generateRecordId
     * @return
     */
    GenerateRecordDO getGenerateRecordByRecordId(String generateRecordId);

}
