package cn.iocoder.yudao.module.identity.service.code;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;
import cn.iocoder.yudao.module.identity.controller.admin.code.vo.CodeCreateReqVO;
import cn.iocoder.yudao.module.identity.controller.admin.code.vo.CodeExportReqVO;
import cn.iocoder.yudao.module.identity.controller.admin.code.vo.CodePageReqVO;
import cn.iocoder.yudao.module.identity.controller.admin.code.vo.CodeUpdateReqVO;
import cn.iocoder.yudao.module.identity.dal.dataobject.code.CodeDO;
import cn.iocoder.yudao.module.identity.dal.mysql.code.CodeMapper;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.context.annotation.Import;

import javax.annotation.Resource;
import java.util.Arrays;

import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.cloneIgnoreId;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertPojoEquals;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertServiceException;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomLongId;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomPojo;
import static cn.iocoder.yudao.module.identity.enums.ErrorCodeConstants.CODE_NOT_EXISTS;
import static org.junit.jupiter.api.Assertions.*;

/**
* {@link CodeServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(CodeServiceImpl.class)
public class CodeServiceImplTest extends BaseDbUnitTest {

    @Resource
    private CodeServiceImpl codeService;

    @Resource
    private CodeMapper codeMapper;

//    @Test
//    public void testCreateCode_success() {
//        // 准备参数
//        CodeCreateReqVO reqVO = randomPojo(CodeCreateReqVO.class);
//
//        // 调用
//        Long codeId = codeService.createCode(reqVO);
//        // 断言
//        assertNotNull(codeId);
//        // 校验记录的属性是否正确
//        CodeDO code = codeMapper.selectById(codeId);
//        assertPojoEquals(reqVO, code);
//    }

    @Test
    public void testCreateCodes_success() {
        // 准备参数
        CodeCreateReqVO reqVO = randomPojo(CodeCreateReqVO.class);

        // 调用
        Long[] codeIds = codeService.createCode(reqVO);
        // 断言
        assertNotNull(codeIds);
        // 校验记录的属性是否正确
        CodeDO code = codeMapper.selectById(Arrays.stream(codeIds).findFirst().isPresent());
        assertPojoEquals(reqVO, code);
    }

    @Test
    public void testUpdateCode_success() {
        // mock 数据
        CodeDO dbCode = randomPojo(CodeDO.class);
        codeMapper.insert(dbCode);// @Sql: 先插入出一条存在的数据
        // 准备参数
        CodeUpdateReqVO reqVO = randomPojo(CodeUpdateReqVO.class, o -> {
            o.setId(dbCode.getId()); // 设置更新的 ID
        });

        // 调用
        codeService.updateCode(reqVO);
        // 校验是否更新正确
        CodeDO code = codeMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, code);
    }

    @Test
    public void testUpdateCode_notExists() {
        // 准备参数
        CodeUpdateReqVO reqVO = randomPojo(CodeUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> codeService.updateCode(reqVO), CODE_NOT_EXISTS);
    }

    @Test
    public void testDeleteCode_success() {
        // mock 数据
        CodeDO dbCode = randomPojo(CodeDO.class);
        codeMapper.insert(dbCode);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbCode.getId();

        // 调用
        codeService.deleteCode(id);
       // 校验数据不存在了
       assertNull(codeMapper.selectById(id));
    }

    @Test
    public void testDeleteCode_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> codeService.deleteCode(id), CODE_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetCodePage() {
       // mock 数据
       CodeDO dbCode = randomPojo(CodeDO.class, o -> { // 等会查询到
           o.setGenerateRecordId(null);
           o.setIdentityCode(null);
           o.setVerfiyCode(null);
           o.setRuleId(null);
           o.setCodeRuleFormat(null);
           o.setCodeType(null);
           o.setCharLength(null);
           o.setCharType(null);
           o.setBatchId(null);
           o.setBatchType(null);
           o.setGoodsId(null);
           o.setGoodsName(null);
           o.setDealerName(null);
           o.setDealerId(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
           o.setCreateTime(null);
           o.setScanCount(null);
           o.setLimitCount(null);
       });
       codeMapper.insert(dbCode);
       // 测试 generateRecordId 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setGenerateRecordId(null)));
       // 测试 identityCode 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setIdentityCode(null)));
       // 测试 verfiyCode 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setVerfiyCode(null)));
       // 测试 ruleId 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setRuleId(null)));
       // 测试 codeRuleFormat 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setCodeRuleFormat(null)));
       // 测试 codeType 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setCodeType(null)));
       // 测试 charLength 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setCharLength(null)));
       // 测试 charType 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setCharType(null)));
       // 测试 batchId 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setBatchId(null)));
       // 测试 batchType 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setBatchType(null)));
       // 测试 goodsId 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setGoodsId(null)));
       // 测试 goodsName 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setGoodsName(null)));
       // 测试 dealerName 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setDealerName(null)));
       // 测试 dealerId 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setDealerId(null)));
       // 测试 belongBusinessId 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setBelongBusinessName(null)));
       // 测试 createTime 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setCreateTime(null)));
       // 测试 scanCount 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setScanCount(null)));
       // 测试 limitCount 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setLimitCount(null)));
       // 准备参数
       CodePageReqVO reqVO = new CodePageReqVO();
       reqVO.setGenerateRecordId(null);
       reqVO.setIdentityCode(null);
       reqVO.setVerfiyCode(null);
       reqVO.setRuleId(null);
       reqVO.setCodeRuleFormat(null);
       reqVO.setCodeType(null);
       reqVO.setCharLength(null);
       reqVO.setCharType(null);
       reqVO.setBatchId(null);
       reqVO.setBatchType(null);
       reqVO.setGoodsId(null);
       reqVO.setGoodsName(null);
       reqVO.setDealerName(null);
       reqVO.setDealerId(null);
       reqVO.setBelongBusinessId(null);
       reqVO.setBelongBusinessName(null);
//       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setScanCount(null);
       reqVO.setLimitCount(null);

       // 调用
       PageResult<CodeDO> pageResult = codeService.getCodePage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbCode, pageResult.getList().get(0));
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetCodeList() {
       // mock 数据
       CodeDO dbCode = randomPojo(CodeDO.class, o -> { // 等会查询到
           o.setGenerateRecordId(null);
           o.setIdentityCode(null);
           o.setVerfiyCode(null);
           o.setRuleId(null);
           o.setCodeRuleFormat(null);
           o.setCodeType(null);
           o.setCharLength(null);
           o.setCharType(null);
           o.setBatchId(null);
           o.setBatchType(null);
           o.setGoodsId(null);
           o.setGoodsName(null);
           o.setDealerName(null);
           o.setDealerId(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
           o.setCreateTime(null);
           o.setScanCount(null);
           o.setLimitCount(null);
       });
       codeMapper.insert(dbCode);
       // 测试 generateRecordId 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setGenerateRecordId(null)));
       // 测试 identityCode 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setIdentityCode(null)));
       // 测试 verfiyCode 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setVerfiyCode(null)));
       // 测试 ruleId 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setRuleId(null)));
       // 测试 codeRuleFormat 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setCodeRuleFormat(null)));
       // 测试 codeType 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setCodeType(null)));
       // 测试 charLength 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setCharLength(null)));
       // 测试 charType 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setCharType(null)));
       // 测试 batchId 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setBatchId(null)));
       // 测试 batchType 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setBatchType(null)));
       // 测试 goodsId 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setGoodsId(null)));
       // 测试 goodsName 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setGoodsName(null)));
       // 测试 dealerName 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setDealerName(null)));
       // 测试 dealerId 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setDealerId(null)));
       // 测试 belongBusinessId 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setBelongBusinessName(null)));
       // 测试 createTime 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setCreateTime(null)));
       // 测试 scanCount 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setScanCount(null)));
       // 测试 limitCount 不匹配
       codeMapper.insert(cloneIgnoreId(dbCode, o -> o.setLimitCount(null)));
       // 准备参数
       CodeExportReqVO reqVO = new CodeExportReqVO();
       reqVO.setGenerateRecordId(null);
       reqVO.setIdentityCode(null);
       reqVO.setVerfiyCode(null);
       reqVO.setRuleId(null);
       reqVO.setCodeRuleFormat(null);
       reqVO.setCodeType(null);
       reqVO.setCharLength(null);
       reqVO.setCharType(null);
       reqVO.setBatchId(null);
       reqVO.setBatchType(null);
       reqVO.setGoodsId(null);
       reqVO.setGoodsName(null);
       reqVO.setDealerName(null);
       reqVO.setDealerId(null);
       reqVO.setBelongBusinessId(null);
       reqVO.setBelongBusinessName(null);
//       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setScanCount(null);
       reqVO.setLimitCount(null);

       // 调用
//       List<CodeDO> list = codeService.getCodeList(reqVO);
       // 断言
//       assertEquals(1, list.size());
//       assertPojoEquals(dbCode, list.get(0));
    }

}
