package cn.iocoder.yudao.module.product.service.envdata;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.product.controller.admin.envdata.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.envdata.EnvDataDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 环境数据 Service 接口
 *
 * @author 芋道源码
 */
public interface EnvDataService {

    /**
     * 创建环境数据
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createEnvData(@Valid EnvDataCreateReqVO createReqVO);

    /**
     * 更新环境数据
     *
     * @param updateReqVO 更新信息
     */
    void updateEnvData(@Valid EnvDataUpdateReqVO updateReqVO);

    /**
     * 删除环境数据
     *
     * @param id 编号
     */
    void deleteEnvData(Long id);

    /**
     * 获得环境数据
     *
     * @param id 编号
     * @return 环境数据
     */
    EnvDataDO getEnvData(Long id);

    /**
     * 获得环境数据分页
     *
     * @param pageReqVO 分页查询
     * @return 环境数据分页
     */
    PageResult<EnvDataRespVO> getEnvDataPage(EnvDataPageReqVO pageReqVO);

    /**
     * 通过emqx发送过来的消息保存为环境数据
     * @param topic
     * @param message
     */
    void saveEnvDataByEmqx(String topic, String message);

    /**
     * 根据地块编码获得最新的环境数据
     * @param landId 地块编码
     * @return 环境数据
     */
    List<EnvDataRespVO> gatLastEnvDatas(String landId);

    /**
     * 根据地块编码获得7天内环境数据的均值
     * @param landId 地块编码
     * @return 环境数据
     */
    List<EnvDataRespVO> getAVGEnvDatas(String landId, Integer day, List<Long> typeIds);

}
