package cn.iocoder.yudao.module.product.service.inspection;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.controller.admin.harvestbatch.vo.InspectionSaveReqVO;
import cn.iocoder.yudao.module.product.controller.admin.inspection.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.inspection.InspectionDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 检验检疫 Service 接口
 *
 * @author 芋道源码
 */
public interface InspectionService {

    /**
     * 创建检验检疫
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createInspection(@Valid InspectionCreateReqVO createReqVO);

    /**
     * 更新检验检疫
     *
     * @param updateReqVO 更新信息
     */
    void updateInspection(@Valid InspectionUpdateReqVO updateReqVO);

    /**
     * 删除检验检疫
     *
     * @param id 编号
     */
    void deleteInspection(Long id);

    /**
     * 获得检验检疫
     *
     * @param id 编号
     * @return 检验检疫
     */
    InspectionRespVO getInspection(Long id);

    /**
     * 获得检验检疫列表
     *
     * @param ids 编号
     * @return 检验检疫列表
     */
    List<InspectionDO> getInspectionList(Collection<Long> ids);

    /**
     * 获得检验检疫分页
     *
     * @param pageReqVO 分页查询
     * @return 检验检疫分页
     */
    PageResult<InspectionDO> getInspectionPage(InspectionPageReqVO pageReqVO);

    /**
     * 获得检验检疫列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 检验检疫列表
     */
    List<InspectionDO> getInspectionList(InspectionExportReqVO exportReqVO);

    /**
     * 上链
     * @param id 主键ID
     */
    void upChain(Long id);

    /**
     * 更新链状态
     * @param id id
     * @param state 状态
     */
    void updateChainStatus(Long id, Integer state);

    /**
     * 根据批次编码获得检验检疫（返回以区块证书的数据格式）
     * @param harvestBatchId 批次编码
     * @return 检验检疫
     */
    List<InspectionRespVO> getByHarvestBatchId(String harvestBatchId);

}
