package cn.iocoder.yudao.module.product.service.deviceinfo;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;
import cn.iocoder.yudao.module.product.controller.admin.deviceinfo.vo.DeviceInfoCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.deviceinfo.vo.DeviceInfoUpdateReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.deviceinfo.DeviceInfoDO;
import cn.iocoder.yudao.module.product.dal.mysql.deviceinfo.DeviceInfoMapper;
import org.junit.jupiter.api.Test;
import org.springframework.context.annotation.Import;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertPojoEquals;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertServiceException;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomLongId;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomPojo;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.DEVICE_INFO_NOT_EXISTS;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;

/**
* {@link DeviceInfoServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(DeviceInfoServiceImpl.class)
public class DeviceInfoServiceImplTest extends BaseDbUnitTest {

    @Resource
    private DeviceInfoServiceImpl deviceInfoService;

    @Resource
    private DeviceInfoMapper deviceInfoMapper;

    @Test
    public void testCreateDeviceInfo_success() {
        // 准备参数
        DeviceInfoCreateReqVO reqVO = randomPojo(DeviceInfoCreateReqVO.class);

        // 调用
        Long deviceInfoId = deviceInfoService.createDeviceInfo(reqVO);
        // 断言
        assertNotNull(deviceInfoId);
        // 校验记录的属性是否正确
        DeviceInfoDO deviceInfo = deviceInfoMapper.selectById(deviceInfoId);
        assertPojoEquals(reqVO, deviceInfo);
    }

    @Test
    public void testUpdateDeviceInfo_success() {
        // mock 数据
        DeviceInfoDO dbDeviceInfo = randomPojo(DeviceInfoDO.class);
        deviceInfoMapper.insert(dbDeviceInfo);// @Sql: 先插入出一条存在的数据
        // 准备参数
        DeviceInfoUpdateReqVO reqVO = randomPojo(DeviceInfoUpdateReqVO.class, o -> {
            o.setId(dbDeviceInfo.getId()); // 设置更新的 ID
        });

        // 调用
        deviceInfoService.updateDeviceInfo(reqVO);
        // 校验是否更新正确
        DeviceInfoDO deviceInfo = deviceInfoMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, deviceInfo);
    }

    @Test
    public void testUpdateDeviceInfo_notExists() {
        // 准备参数
        DeviceInfoUpdateReqVO reqVO = randomPojo(DeviceInfoUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> deviceInfoService.updateDeviceInfo(reqVO), DEVICE_INFO_NOT_EXISTS);
    }

    @Test
    public void testDeleteDeviceInfo_success() {
        // mock 数据
        DeviceInfoDO dbDeviceInfo = randomPojo(DeviceInfoDO.class);
        deviceInfoMapper.insert(dbDeviceInfo);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbDeviceInfo.getId();

        // 调用
        deviceInfoService.deleteDeviceInfo(id);
       // 校验数据不存在了
       assertNull(deviceInfoMapper.selectById(id));
    }

    @Test
    public void testDeleteDeviceInfo_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> deviceInfoService.deleteDeviceInfo(id), DEVICE_INFO_NOT_EXISTS);
    }

}
