package cn.iocoder.yudao.module.product.service.envdata;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;

import cn.iocoder.yudao.module.product.controller.admin.envdata.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.envdata.EnvDataDO;
import cn.iocoder.yudao.module.product.dal.mysql.envdata.EnvDataMapper;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.*;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.*;
import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.*;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
* {@link EnvDataServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(EnvDataServiceImpl.class)
public class EnvDataServiceImplTest extends BaseDbUnitTest {

    @Resource
    private EnvDataServiceImpl envDataService;

    @Resource
    private EnvDataMapper envDataMapper;

    @Test
    public void testCreateEnvData_success() {
        // 准备参数
        EnvDataCreateReqVO reqVO = randomPojo(EnvDataCreateReqVO.class);

        // 调用
        Long envDataId = envDataService.createEnvData(reqVO);
        // 断言
        assertNotNull(envDataId);
        // 校验记录的属性是否正确
        EnvDataDO envData = envDataMapper.selectById(envDataId);
        assertPojoEquals(reqVO, envData);
    }

    @Test
    public void testUpdateEnvData_success() {
        // mock 数据
        EnvDataDO dbEnvData = randomPojo(EnvDataDO.class);
        envDataMapper.insert(dbEnvData);// @Sql: 先插入出一条存在的数据
        // 准备参数
        EnvDataUpdateReqVO reqVO = randomPojo(EnvDataUpdateReqVO.class, o -> {
            o.setId(dbEnvData.getId()); // 设置更新的 ID
        });

        // 调用
        envDataService.updateEnvData(reqVO);
        // 校验是否更新正确
        EnvDataDO envData = envDataMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, envData);
    }

    @Test
    public void testUpdateEnvData_notExists() {
        // 准备参数
        EnvDataUpdateReqVO reqVO = randomPojo(EnvDataUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> envDataService.updateEnvData(reqVO), ENV_DATA_NOT_EXISTS);
    }

    @Test
    public void testDeleteEnvData_success() {
        // mock 数据
        EnvDataDO dbEnvData = randomPojo(EnvDataDO.class);
        envDataMapper.insert(dbEnvData);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbEnvData.getId();

        // 调用
        envDataService.deleteEnvData(id);
       // 校验数据不存在了
       assertNull(envDataMapper.selectById(id));
    }

    @Test
    public void testDeleteEnvData_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> envDataService.deleteEnvData(id), ENV_DATA_NOT_EXISTS);
    }

}
