package cn.iocoder.yudao.module.system.service.languagetype;

import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.languagetype.vo.LanguageTypeCreateReqVO;
import cn.iocoder.yudao.module.system.controller.admin.languagetype.vo.LanguageTypeExportReqVO;
import cn.iocoder.yudao.module.system.controller.admin.languagetype.vo.LanguageTypePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.languagetype.vo.LanguageTypeUpdateReqVO;
import cn.iocoder.yudao.module.system.convert.languagetype.LanguageTypeConvert;
import cn.iocoder.yudao.module.system.dal.dataobject.languagetype.LanguageTypeDO;
import cn.iocoder.yudao.module.system.dal.mysql.languagetype.LanguageTypeMapper;
import cn.iocoder.yudao.module.system.enums.ErrorCodeConstants;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

/**
 * 语言类型 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class LanguageTypeServiceImpl implements LanguageTypeService {

    @Resource
    private LanguageTypeMapper languageTypeMapper;

    @Override
    public Long createLanguageType(LanguageTypeCreateReqVO createReqVO) {
        // 插入
        LanguageTypeDO languageType = LanguageTypeConvert.INSTANCE.convert(createReqVO);
        languageTypeMapper.insert(languageType);
        // 返回
        return languageType.getId();
    }

    @Override
    public void updateLanguageType(LanguageTypeUpdateReqVO updateReqVO) {
        // 校验存在
        validateLanguageTypeExists(updateReqVO.getId());
        // 更新
        LanguageTypeDO updateObj = LanguageTypeConvert.INSTANCE.convert(updateReqVO);
        languageTypeMapper.updateById(updateObj);
    }

    @Override
    public void deleteLanguageType(Long id) {
        // 校验存在
        validateLanguageTypeExists(id);
        // 删除
        languageTypeMapper.deleteById(id);
    }

    private void validateLanguageTypeExists(Long id) {
        if (languageTypeMapper.selectById(id) == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.LANGUAGE_TYPE_NOT_EXISTS);
        }
    }

    @Override
    public LanguageTypeDO getLanguageType(Long id) {
        return languageTypeMapper.selectById(id);
    }

    @Override
    public List<LanguageTypeDO> getLanguageTypeList(Collection<Long> ids) {
        return languageTypeMapper.selectBatchIds(ids);
    }

    @Override
    public List<LanguageTypeDO> getAllLanguageTypeList() {
        return languageTypeMapper.selectList();
    }

    @Override
    public PageResult<LanguageTypeDO> getLanguageTypePage(LanguageTypePageReqVO pageReqVO) {
        return languageTypeMapper.selectPage(pageReqVO);
    }

    @Override
    public List<LanguageTypeDO> getLanguageTypeList(LanguageTypeExportReqVO exportReqVO) {
        return languageTypeMapper.selectList(exportReqVO);
    }

}
